

  UNIT BIPOS;

   {Declarations des unites utilisees dans le programme principal}

                            INTERFACE

   uses constype,wincrt,fonvect,fonscal;

   {Ce logiciel determine la trajectoire keplerienne joignant deux points
   donnes dans le repere j2000 par leurs coordonnees avec les dates depart
   et arrivee connues.

   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
   *  Notations utilisees pour les variables d'entre:                    *
   *                                                                      *
   *                                                                      *
   *    ARRIVEE   ra---->ra[1],ra[2],ra[3]                                *
   *                                                                      *
   *                                                                      *
   *    DEPART    rd---->rd[1],rd[2],rd[3]                                *
   *                                                                      *
   *    DUREE:Duree du voyage en jours(Utiliser JOURJULI)                                                                  *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **

   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
   *   Notations:BIPOS permettra de calculer la vitesse aux 2 points de   *
   *             depart et d'arrivee,le calcul s'acheve avec RVPARAM      *
   *    CAS:indique le cas de vol.                                        *
   *                                                                      *
   *                                                                      *
   *    VITDEPART:valeur en KM/S de la vitesse au depart dans J2000.      *
   *    VITARRIVEE:valeur en KM/S de la vitesse  l'arrivee dans J2000.   *
   *                                                                      *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
       NB:coordonnees et distance en km
          angles en radians dans les calculs et degres en sortie even-
          tuelle.
       NB:La constante de gravitation mu est soit
                                  mus pour le soleil
                                  mut pour la terre
                                  etc.....
          Les valeurs sont donnees dans l'unit CONSTYPE }


PROCEDURE ELLIPSE2POINTS(rd,ra:vecteur;duree:real;mu:real;var cas:integer;
                   var vitdepart,vitarrivee:vecteur);

                           IMPLEMENTATION
 PROCEDURE ELLIPSE2POINTS;

    VAR x,y,w:vecteur;
        p,ah,eh,ch,psia,psib,psiasymptote,alpha:real;
        ae,ce,exc,fid,fia,sensdirect,psilim1,dureecalculee:real;
        chr:char;

        conclusion:boolean;
       
   {*************************************************************************}

    PROCEDURE HYPERBOLE;

    const k:vecteur=(0,0,1);

    var mrd,result:vecteur;

   BEGIN

    {Mise en place des axes x et y unitaires}

      SCALVECT(-1,rd,mrd);

      SOMME(ra,mrd,result);

      UNITAIRE(result,x);

    {Calcul de w:unitaire normal au plan orbital et induisant le
     mouvement}

      PRODVECT(rd,ra,w);

    {Test de non colinearite des rayons vecteurs depart et arrivee}

      alpha:=ANGVECT(rd,ra);
      if((alpha<1e-3) or (abs(pi-alpha)<1e-3)) then
      write('Theorie non applicable:trajectoire non unique');

      UNITAIRE(w,w);
      sensdirect:=1;
      if (PRODSCAL(w,k)<0) then
         begin
         SCALVECT(-1,w,w);
         sensdirect:=-1;
         end;
      PRODVECT(w,x,y);

    {Mise en place de l hyperbole,ch 1/2 axe focal,ah 1/2 grand-axe
     eh:excentricite,p parametre}

      ch:=NORM(result)/2;
      ah:=abs((NORM(ra)-NORM(rd)))/2;
      eh:=ch/ah;
      p:=ah*(eh*eh-1);

    {Calcul du 1/2 angle d ouverture des asymptotes et des angles en a,d,o }

      psiasymptote:=ARCCOS(1/eh);
   {Calcul des angles entre 0 et pi ,le cosinus est parfait}

      psia:=ARCCOS((4*ch*ch+Norm(rd)*Norm(rd)-Norm(ra)*Norm(ra))/4/ch/Norm(rd));
      psib:=pi-alpha-psia;

   END;

{***************************************************************************}

   PROCEDURE ELLIPSE(psi:real);

    var  rho,xo1,yo1,xo,yo:real;

    BEGIN

      {Calcul du 1/2 grand-axe ae,de la 1/2 distance focale ce,du rayon vecteur
       ro1 du second foyer de l ellipse,de l excentricite exc}

      if (NORM(ra)>Norm(rd)) then
       begin
        rho:=p/(1+eh*cos(psi));

        {Calcul des coordonnees de o1 dans les axes x et y}

         xo1:=ch-rho*cos(psi);
         yo1:=rho*sin(psi);

         {Calcul du demi grand-axe de l ellipse}

         ae:=(sqrt((xo1+ch)*(xo1+ch)+yo1*yo1)+NORM(rd))/2;

         if sensdirect<0 then psib:=-psib;

         {Calcul de la demi distance focale de l ellipse=OO1/2}
         xo:=ch-NORM(ra)*cos(psib);
         yo:=norm(ra)*sin(psib);
         ce:=sqrt((xo1-xo)*(xo1-xo)+(yo1-yo)*(yo1-yo))/2;
              exc:=ce/ae;
         if sensdirect<0 then psib:=-psib;

      end;

    if (NORM(ra)<Norm(rd)) then
      begin
        rho:=p/(1+eh*cos(psi));

       {Calcul des coordonnees de o1 dans les axes x et y}

       xo1:=-ch+rho*cos(psi);
       yo1:=rho*sin(psi);

       {Calcul du demi grand-axe de l ellipse}

       ae:=(sqrt((xo1+ch)*(xo1+ch)+yo1*yo1)+NORM(rd))/2;

       if sensdirect<0 then psib:=-psib;

       {Calcul de la demi distance focale de l ellipse=OO1/2}
       xo:=ch-NORM(ra)*cos(psib);
       yo:=norm(ra)*sin(psib);
       ce:=sqrt((xo1-xo)*(xo1-xo)+(yo1-yo)*(yo1-yo))/2;
            exc:=ce/ae;
       if sensdirect<0 then psib:=-psib;

    end;

   END;

{***************************************************************************}
 FUNCTION test(ra,rd:vecteur):boolean;
         var K1:real;
             p1,p2,punit:vecteur;
          begin
            K1:=(sin(fia)*(cos(fid)-exc)- sin(fid)*(cos(fia)-exc))*ae;
            SCALVECT(sin(fia),rd,p1);
            SCALVECT(-sin(fid),ra,p2);
            SOMME(p1,p2,punit);
            SCALVECT(1/k1,punit,punit);
            test:= (abs(norm(punit)-1)<0.01);
          end;

 {*************************************************************************}


  PROCEDURE TEMPS(var ae,exc,t,fid,fia:real);

   var tempo:real;

   {Calcul des anomalies excentriques fid au depart et fia a l arrivee}

   BEGIN

   fid:=ARCCOS((1-NORM(rd)/ae)/exc);
   fia:=ARCCOS((1-NORM(ra)/ae)/exc);

   {Calcul du temps de voyage en jours suivant les cas}

   tempo:=ae*sqrt(ae/mus);

    case  cas of
           1:t:=tempo*(fia-fid-exc*(sin(fia)-sin(fid)));
           2:begin
	       t:=tempo*(fid+fia-exc*(sin(fid)+sin(fia)));
               fid:=2*pi-fid;
	      end;
           3:begin
	       t:=tempo*(2*pi-fid-fia+exc*(sin(fid)+sin(fia)));
               fia:=2*pi-fia;
             end;
           4:begin
               t:=tempo*(fid+fia-exc*(sin(fid)+sin(fia)));
               fid:=2*pi-fid;
               end;
           5:begin
               t:=tempo*(2*pi+fid-fia-exc*(sin(fid)-sin(fia)));
               fia:=2*pi-fia;
               fid:=2*pi-fid;
             end;
           6:begin
               t:=tempo*(2*pi-fid-fia-exc*(sin(fia)-sin(fid)));
               fia:=2*pi-fia;
             end;
           7:begin
               t:=tempo*(fia+fid-exc*(sin(fia)+sin(fid)));
               fid:=2*pi-fid;
             end;
           8:begin
               t:=tempo*(fid-fia-exc*(sin(fid)-sin(fia)));
               fid:=2*pi-fid;
               fia:=2*pi-fia;
             end;
           9:begin
               t:=tempo*(2*pi-fia-fid+exc*(sin(fia)+sin(fid)));
               fia:=2*pi-fia;
             end;
           10:begin
               t:=tempo*(2*pi-fid-fia+exc*(sin(fid)+sin(fia)));
               fia:=2*pi-fia;
              end;
           11:t:=tempo*(2*pi-fid+fia+exc*(sin(fid)-sin(fia)));
           12:begin
                t:=tempo*(fia+fid-exc*(sin(fia)+sin(fid)));
                fid:=2*pi-fid;
              end;

       end;
  END;
{**************************************************************************}

  PROCEDURE ANGLELIMITEMONTANT(var psilim1:real);

 var k,xa,xb,xc,ki1,ki2,psi1,psi2:real;

  BEGIN

  if(NORM(ra)>NORM(rd)) then
   begin
     k:=2*eh/(eh*eh-1);
     xa:=sin(psia)-k*eh;
     xb:=cos(psia);
     xc:=sqrt(xa*xa+xb*xb);
     ki1:=ARCSIN(xa/xc);
     ki2:=ARCSIN(k/xc);
     psi1:=ki2-ki1;
     psi2:=pi-ki2-ki1;

     if ((psi1>psib) and (psi1<pi-psiasymptote)) then psilim1:=psi1;
     if ((psi2>psib) and (psi2<pi-psiasymptote)) then psilim1:=psi2;
   end;

  if (NORM(ra)<NORM(rd)) then
   begin

     k:=2*eh/(eh*eh-1);
     xa:=sin(psib)-k*eh;
     xb:=cos(psib);
     xc:=sqrt(xa*xa+xb*xb);
     ki1:=ARCSIN(xa/xc);
     ki2:=ARCSIN(k/xc);
     psi1:=ki2-ki1;
     psi2:=pi-ki2-ki1;

     if ((psi1>psia) and (psi1<pi-psiasymptote)) then psilim1:=psi1;
     if ((psi2>psia) and (psi2<pi-psiasymptote)) then psilim1:=psi2;
  end;

  END;


{*************************************************************************}

 PROCEDURE ITERATION(psimin,psimax:real;var conclusion:boolean);

    label lab1,lab2,lab3;

   var psi,tmin,tmax,t,deltat,deltapsi:real;
       i:integer;
       tvoy:array[0..51] of real;
       chr:char;

      BEGIN
        {Tant que conclusion =false la solution n est pas trouvee}

        conclusion:=false;

     {INITIALISATION DE LA BOUCLE DE CALCUL DE LA SOLUTION}

     lab1:deltapsi:=(psimax-psimin)/50;
          i:=1;

     lab2:begin
          {Calcul de la duree en debut d intervalle }

               psi:=psimin+(i-1)*deltapsi;
               ellipse(psi);
               temps(ae,exc,tvoy[i],fid,fia);
               if(abs(duree-tvoy[i])<0.1) then
                begin
                  dureecalculee:=tvoy[i];
                  conclusion:=true;
                  goto lab3;
                end;
          {Calcul de la duree en fin de l intervalle}

               psi:=psimin+i*deltapsi;
               ellipse(psi);
               temps(ae,exc,tvoy[i+1],fid,fia);

               if(abs(duree-tvoy[i+1])<0.1) then
                begin
                  dureecalculee:=tvoy[i+1];
                  conclusion:=true;
                  goto lab3;
                end;
          {Condition de changement de signe:solution localisee}
              if ((duree-tvoy[i])*(duree-tvoy[i+1])<0) then
                   begin
                     psimax:=psi;
                     psimin:=psi-deltapsi;
                     goto lab1;
                   end;

             end;
                i:=i+1;
                if (i<51) then goto lab2;

      lab3:END;

 {*************************************************************************}

  PROCEDURE VITESSEDEPARTARRIVEE;

  {Cette procedure calcule aupoint d de depart la vitesse de la sonde,par
  ses composantes dans le repere J2000 en km/s.
   On pourra donc ensuite avec l unite RVPARAM calculer les parametres
	 orbitaux de la sonde}

   var  k1:real;
        p1,p2,punit,qunit:vecteur;
   BEGIN
       K1:=(sin(fia)*(cos(fid)-exc)- sin(fid)*(cos(fia)-exc))*ae;
       SCALVECT(sin(fia),rd,p1);
       SCALVECT(-sin(fid),ra,p2);
       SOMME(p1,p2,punit);
       SCALVECT(1/k1,punit,punit);
       PRODVECT(w,punit,qunit);

    {Calcul de la vitesse de depart(voir cnes p 62)}

       k1:=-sqrt(mus/(ae*sqr(ae)))*sin(fid)*ae/(1-exc*cos(fid));
       SCALVECT(k1,punit,p1);
       k1:=sqrt(mus/(ae*sqr(ae)))*cos(fid)*sqrt(1-exc*exc)*ae/
			       (1-exc*cos(fid));
       SCALVECT(k1,qunit,p2);
       SOMME(p1,p2,vitdepart);

    {Calcul de la vitesse d arrivee(voir cnes p 62)}

       k1:=-sqrt(mus/(ae*sqr(ae)))*sin(fia)*ae/(1-exc*cos(fia));
       SCALVECT(k1,punit,p1);
       k1:=sqrt(mus/(ae*sqr(ae)))*cos(fia)*sqrt(1-exc*exc)*ae/
			       (1-exc*cos(fia));
       SCALVECT(k1,qunit,p2);
       SOMME(p1,p2,vitarrivee);

   END;
 {**************************************************************************}

  PROCEDURE RESULTATS;
   BEGIN
      VITESSEDEPARTARRIVEE;
     END;
{***************************************************************************}

 PROCEDURE SEPARATION_CAS_DE_MONTEE;

 LABEL lab4;

 VAR angle,psiinf,psisup,xra,xrd:real;
     test_P:boolean;
  BEGIN
    xra:=norm(ra);
    xrd:=norm(rd);

   {Calcul de l'angle maximum sur psi qui limite la distance AO1  5 E9 KM}
    angle:=0.99*(pi-psiasymptote);

    psiinf:=-angle;
    psisup:=angle;

    if ((xra>xrd) and (psia<psiasymptote) and (sensdirect>0))
       then
          begin
           ANGLELIMITEMONTANT(psilim1);
           if (psilim1<psisup)
                then
                   begin
                      cas:=2;
                      ITERATION(psilim1,psisup,conclusion);
                      test_P:=test(ra,rd);
                      conclusion:=conclusion and test_P;
                      if conclusion then
                        begin
                         RESULTATS;
                         goto lab4;
                        end;
                   end;
	    end;

    {Le cas 2 est epuise,on envisage les autres cas}

    if (xra>xrd) and (sensdirect>0) then
       begin
         cas:=1;
         ITERATION(psib,psisup,conclusion);
         test_P:=test(ra,rd);
         conclusion:=conclusion and test_P;
         if conclusion then
            begin
	      RESULTATS;
	      goto lab4;
            end;
       end;

     if (xra>xrd) and (sensdirect>0) then
       begin
         cas:=4;
         ITERATION(psib,psisup,conclusion);
         test_P:=test(ra,rd);
         conclusion:=conclusion and test_P;
         if conclusion then
            begin
	      RESULTATS;
	      goto lab4;
            end;
       end;

       if(xra>xrd) and (sensdirect>0) then
         begin
           cas:=3;
           ITERATION(psiinf,psib,conclusion);
           test_P:=test(ra,rd);
           conclusion:=conclusion and test_P;
           if conclusion then
             begin
               RESULTATS;
	       goto lab4;
             end;
           if not(conclusion) then goto lab4;

         end;
                       {************************}

    if ((xra>xrd) and (psia<psiasymptote) and (sensdirect<0))
       then
          begin
           ANGLELIMITEMONTANT(psilim1);
           if (psilim1<psisup)
                then
                   begin
                      cas:=6;
                      ITERATION(psiinf,-psilim1,conclusion);
                      test_P:=test(ra,rd);
                      conclusion:=conclusion and test_P;
                      if conclusion
                         then
                          begin
			    RESULTATS;
			    goto lab4;
                         end;
		    end;
         end;

    {Le cas 6 est epuise,on envisage les autres cas}

    if (xra>xrd) and (sensdirect<0) then
       begin
         cas:=5;
         ITERATION(psiinf,-psib,conclusion);
         test_P:=test(ra,rd);
         conclusion:=conclusion and test_P;
         if conclusion then
            begin
              RESULTATS;
              goto lab4;

           end;
       end;

       if(xra>xrd) and (sensdirect<0) then
         begin
           cas:=4;
           ITERATION(-psib,psisup,conclusion);
           test_P:=test(ra,rd);
            conclusion:=conclusion and test_P;
           if conclusion then
	     begin
               RESULTATS;
               goto lab4;
             end;
           if not(conclusion) then goto lab4;

	 end;


  lab4:END;
  {***********************************************************************}

  PROCEDURE SEPARATION_CAS_DE_DESCENTE;

 LABEL lab5;

 VAR angle,psiinf,psisup,xra,xrd:real;
     test_P:boolean;
 BEGIN
    xra:=norm(ra);
    xrd:=norm(rd);
    {Calcul de l'angle maximum sur psi qui limite la distance AO1  5 E9 KM}
    angle:=0.99*(pi-psiasymptote);

    psiinf:=-angle;
    psisup:=angle;

    if ((xra<xrd) and (psib<psiasymptote) and (sensdirect>0))
       then
          begin
           ANGLELIMITEMONTANT(psilim1);
           if (psilim1<psisup)
                then
                   begin
                      cas:=7;
                      ITERATION(psilim1,psisup,conclusion);
                      test_P:=test(ra,rd);
                      conclusion:=conclusion and test_P;
                      if conclusion
                         then
                          begin
                            RESULTATS;
                            goto lab5;
                         end;
                  end;
          end;

    {Le cas 7 est epuise,on envisage les autres cas}

    if (xra<xrd) and (sensdirect>0) then
       begin
         cas:=8;
         ITERATION(psia,psisup,conclusion);
         test_P:=test(ra,rd);
           conclusion:=conclusion and test_P;
         if conclusion then
            begin
              RESULTATS;
              goto lab5;

           end;
       end;

       if(xra<xrd) and (sensdirect>0) then
         begin
           cas:=9;
           ITERATION(psiinf,psib,conclusion);
           test_P:=test(ra,rd);
           conclusion:=conclusion and test_P;
           if conclusion then
             begin
               RESULTATS;
               goto lab5;
             end;
           if not(conclusion) then goto lab5;

         end;
                       {************************}

    if ((xra<xrd) and (psib<psiasymptote) and (sensdirect<0))
       then
          begin
           ANGLELIMITEMONTANT(psilim1);
           if (psilim1<psisup)
                then
                   begin
                      cas:=10;
                      ITERATION(psiinf,-psilim1,conclusion);
                      test_P:=test(ra,rd);
                      conclusion:=conclusion and test_P;
                      if conclusion
                         then
                          begin
                            RESULTATS;
                            goto lab5;
                         end;
                   end;
          end;

    {Le cas 10 est epuise,on envisage les autres cas}

    if (xra<xrd) and (sensdirect<0) then
       begin
         cas:=11;
         ITERATION(psiinf,-psia,conclusion);
         test_P:=test(ra,rd);
           conclusion:=conclusion and test_P;
         if conclusion then
            begin
              RESULTATS;
              goto lab5;

            end;
       end;

       if(xra<xrd) and (sensdirect<0) then
         begin
           cas:=12;
           ITERATION(-psia,psisup,conclusion);
           test_P:=test(ra,rd);
           conclusion:=conclusion and test_P;
           if conclusion then
             begin
               RESULTATS;
               goto lab5;
             end;
           if not(conclusion) then goto lab5;

         end;

  lab5:END;

  {************************************************************************

                       PROGRAMME PRINCIPAL

  ************************************************************************}

 BEGIN
     clrscr;
     duree:=duree*86400;
     HYPERBOLE;

    if NORM(ra)>NORM(rd)  then SEPARATION_CAS_DE_MONTEE;
    if NORM(ra)<Norm(rd) then SEPARATION_CAS_DE_DESCENTE;


 END;

END.



